%mortgage_vs_N
% Plot graphs of total interest I to mortgage value M0 and monthly payment p
% as a function of number of years N, for various APR values.
%
% LAST UPDATED by Andy French October 2019

function mortgage_vs_N

%Value of mortgage /k
M0 = 250;

%Number of years of mortgage
Nmin = 10; Nmax = 50; n = 100;

%Annual Percentage Rate (APR) or interest (%)
Rmin = 1; Rmax = 10; r = 6;

%Fontsize for graphs
fsize = 18;

%

%Define range of N
N = floor( linspace( Nmin, Nmax, n ) );

%Define range of R
R = floor( linspace( Rmin, Rmax, r ) );

%Calculate fixed rate mortgage profiles
F = zeros(n,r); p = zeros(n,r);
for i = 1:r
    for j=1:n
        %Evaluate fixed rate model
        [F(j,i),I(j,i),M,p(j,i)] = fixedrate( M0, N(j), R(i) );
    end
end

%Plot monthly payment /M0 (%) vs N
N = repmat( N.',[1,r] );
fig = figure('color',[1 1 1]);
plot( N, 100*p/M0, 'linewidth',2 );
xlim([Nmin,Nmax]);
title_str = 'Monthly payment as % frac of M_0';
title( title_str,'fontsize',fsize);
legend_str = {};
for i = 1:r
    legend_str = [legend_str,['R=',num2str(R(i)),'%']];
end
legend( legend_str,'fontsize',fsize,'location','northeast');
set(gca,'fontsize',fsize);
xlabel('Mortgage duration N years','fontsize',fsize)
ylabel('Monthly payment p / M_0 (%)','fontsize',fsize);
grid on;
print( fig, 'p over M0 vs N.png', '-dpng','-r300' );

%

%Plot total interest / initial mortgage value
clf; plot( N, F, 'linewidth',2 );
xlim([Nmin,Nmax]);
title_str = 'Total interest I / M_0';
title( title_str,'fontsize',fsize);
legend( legend_str,'fontsize',fsize,'location','northwest');
set(gca,'fontsize',fsize);
xlabel('Mortgage duration N years','fontsize',fsize)
ylabel('Total interest / M_0','fontsize',fsize);
grid on;
print( fig, 'I over M0 vs N.png', '-dpng','-r300' );
close(fig);

%%

%Fixed rate mortgage calculator
% R Annual Percentage Rate (%) of interest
% M0 Amount borrowed (e.g. in k)
% N number of years of mortgage
% p monthly payment (same units as M0)
% M Vector of mortgage values per month
% I Total interest paid over mortgage term (same units as M0)
% F Total interest/amount borrowed
function [F,I,M,p] = fixedrate( M0, N, R )
k = R/1200;
p = k*M0/( 1 - ( k + 1 )^(-12*N) );
n = 0:12*N;
M = ( (1 + k ).^n )*( M0 - p/k ) + p/k;
I = 12*N*k*M0/( 1 - ( k + 1 )^(-12*N) ) - M0;
F = I/M0;

%%

%Fixed rate mortgage calculator test (uses iteration rather than closed
%form)
function [M,p] = fixedratetest( M0, N, R )
M = [];
k = R/1200;
p = k*M0/( 1 - ( k + 1 )^(-12*N) );
for n=0:12*N
    M = [M, ( (1+k)^n )*( M0 - p/k ) + p/k ];
end

%End of code