%mortgage_vis_time
% Time history of a fixed rate mortgage.
%
% LAST UPDATED by Andy French October 2019

function mortgage_vs_time

%Number of years of mortgage
N = 30;

%Value of mortgage /k
M0 = 250;

%Annual Percentage Rate (APR) or interest (%)
Rmin = 1; Rmax = 10; r = 6;

%Fontsize for graphs
fsize = 18;

%%

%Define time in months
t = repmat( (0:12*N).',[1,r] );

%Define range of R
R = floor( linspace( Rmin, Rmax, r ) );

%Calculate fixed rate mortgage profiles
F = zeros(1,r); I = zeros(1,r); M = zeros(N*12+1,r); p = zeros(1,r);
for i = 1:r
    %Evaluate fixed rate model
    [F(i),I(i),M(:,i),p(i)] = fixedrate( M0, N, R(i) );
    
    %Check closed form calculation with iterative computation
    [FF,II,MM,pp] = fixedrate( M0, N, R(i) );
    checksum = sum( M(:,i).' - MM );
end

%Plot amount owed /k vs time/months
fig = figure('name','mortgage','color',[1 1 1] );
plot( t, M, 'linewidth',2 );
xlim([0,12*N]); ylim([0,M0]);
title_str = ['N=',num2str(N),' years, M_0=',num2str(M0),'k'];
title( title_str,'fontsize',fsize);
legend_str = {};
for i = 1:r
    legend_str = [legend_str,['R=',num2str(R(i)),...
        '%, p=',num2str(p(i)*1000,4),...
        ', I=',num2str(I(i),3),'k']];
end
legend( legend_str,'fontsize',fsize,'location','southwest');
set(gca,'fontsize',fsize);
xlabel('Time /months','fontsize',fsize)
ylabel('Amount owed M (k)','fontsize',fsize);
grid on;
s = ['M vs t N=',num2str(N),'years, M_0=',num2str(M0),'k'];
print( fig, [strrep(s,'.','p'),'.png'], '-dpng','-r300' );
close(gcf);

%%

%Fixed rate mortgage calculator
% R Annual Percentage Rate (%) of interest
% M0 Amount borrowed (e.g. in k)
% N number of years of mortgage
% p monthly payment (same units as M0)
% M Vector of mortgage values per month
% I Total interest paid over mortgage term (same units as M0)
% F Total interest/amount borrowed
function [F,I,M,p] = fixedrate( M0, N, R )
k = R/1200;
p = k*M0/( 1 - ( k + 1 )^(-12*N) );
n = 0:12*N;
M = ( (1 + k ).^n )*( M0 - p/k ) + p/k;
I = 12*N*k*M0/( 1 - ( k + 1 )^(-12*N) ) - M0;
F = I/M0;

%%

%Fixed rate mortgage calculator test (uses iteration rather than closed
%form)
function [M,p] = fixedratetest( M0, N, R )
M = [];
k = R/1200;
p = k*M0/( 1 - ( k + 1 )^(-12*N) );
for n=0:12*N
    M = [M, ( (1+k)^n )*( M0 - p/k ) + p/k ];
end

%End of code