%savings
% Model of a savings (or investment) account offering R % compound 
% interest per annum. Assume an initial deposit of S0 () and a regular monthy
% investment p (). Assume interest is calculated daily, based upon R/365
% % of the daily balance. As per many accounts, interest is added to the
% account at the end of the year. i.e. compound interest only happens
% of a year on year basis, not within years.
%
% LAST UPDATED by Andy French Jan 2019

function savings

%Fontsize for plots
fsize = 18;

%Initial deposit ()
S0 = 1000;

%Monthly investment p ()
p = 500;

%APR (%)
R = 2;

%Number of years of regular savings
N = 20;

%

%Define time array, which is number of days since the initial deposit
t = 0:N*365;

%Initialize value of savings account. Assume no withdrawls!
S = [S0,zeros(1,365*N)];

%Initialize value of investment (no interest).
V = [S0,zeros(1,365*N)];

%Initialize month and year daily counters
m = 0; y = 0;

%Initialize annual interest ()
I = 0;

%Step through time and (i) add monthly payments, (ii) calculate interest,
%(iii) add interest at the end of each year
for n=1:N*365
    
    %Add daily interest
    I = I + (0.01*R/365)*S(n);
    
    %Increment month and year counters, and check if a month or year has
    %passed. If so add payment, or interest respectively
    m = m + 1;
    y = y + 1;
    if m > 365/12
        %Month has elapsed. Reset month counter and add regular saving p
        m = 0;
        S(n+1) = S(n) + p;
        V(n+1) = V(n) + p;
    else
        S(n+1) = S(n);
        V(n+1) = V(n);
    end
    if y > 365
        %Year has passed. reset year counter and add interest accrued I
        %over the year. Then reset I to zero.
        y = 0;
        S(n+1) = S(n+1) + I;
        I = 0;
    end   
end

%Compute overall growth %
g = 100*( S(end)-V(end) )/V(end);

%Plot value of savings (S) and investment (V) vs time t /years
figure('color',[1 1 1],'name','savings');
plot( t/365, S/1000, 'g','linewidth',2); hold on;
plot( t/365, V/1000, 'b','linewidth',2);
grid on
xlabel('Time /years','fontsize',fsize);
ylabel('Balance in k','fontsize',fsize);
legend({'Savings','Investment'},'fontsize',fsize,'location','southeast');
set(gca,'fontsize',fsize);
title_str = ['Savings account. N=',num2str(N),' years, R = ',num2str(R),'%, p=',num2str(p),', g=',num2str(g,3),'%'];
title( title_str,'fontsize',fsize );
s = ['Savings N=',num2str(N),' years, R = ',num2str(R),', p=',num2str(p),', g=',num2str(g,3)];
s = strrep(s,'.','p');
print( gcf, [s,'.png'],'-dpng','-r300' );
close(gcf);

%End of code