%zbroker
%
% A very simple automatic stockbroker algorithm, based upon historic FTSE
% 100 data. All stock prices are in pence sterling. zbroker assumes stock
% data are Normally distributed within a moving window of D days
%
% * Start with a initial fund size
% * Initially divide up fund equally amongst all stocks, weighted by stock price.
% * Assume a transaction loss of L % of stock price. This is assumed to be
%   the same for selling or buying.
% * Assume transactions occur on the same day, i.e. there is no delay on a
%   daily scale.
% * Each day have a transaction fund of T % of current stock portfolio
% * Each day sell maximum T percent of portfolio with Z values above Zsell,
%   and buy from the market those with Z values below Zbuy, up to the value
%   of the cash account.
%   Z = ( stock value - mean value ) / standard deviation
%   Means and standard deviations (volatility) are computed from a moving
%   average (set window size in days) of past FTSE data.
% * If a stock is removed from the FTSE 100 it is immediately sold. Its
%   closing value will be added to the portfolio value.
% * Assumes the total volume of shares is much higher (and will be
%   unaffected by) the buying and selling of stocks. (A separate model will
%   take this into account! i.e. a maximum number of stocks - the volume?)
%
%   Outputs are:
%
%   (i) A graph of portfolio valuation in k vs time /years
%   (i) A matrix of portfolio held vs time /years. Matrix elements are (i)
%   number of shares or (ii) total value.
%
% LAST UPDATED by Andy French October 2019

function zbroker
global p s L T zsell zbuy
t0 = tic;

%Display opening message to command window
disp(' ')
disp(' ***** zbroker FTSE investment simulation ***** ')
disp('            Andy French. Oct 2019 ')
disp(' ');

%Run simulations with different input values
[filename, pathname] = uigetfile('*.m', 'Choose zbroker run script');
fid = fopen( [pathname,filename] ); eval( fscanf(fid, '%c') ); fclose(fid);

%Save portfolio time history to a .mat file
disp(' ... Saving data and plotting graphs')
save portfolio P1 portfolio_value1 portfolio_cash1 P2 portfolio_value2 portfolio_cash2

%Load UK inflation data
load UK_inflation;

%Determine simulation start years from 1/1/1948
load FTSE100_stats;
[num_stocks,num_days] = size(S_interp);
start_year = ( datenum( start_date,'dd-mm-yyyy' ) - datenum('1-1-1948') )/365;

%Calculate equivalent of 1 (1948) in years of simulation
pounds_years = interp1( UK2019.year, UK2019.pounds_1948, t_years + start_year + 1948 )/100;

%Calculate equivalent of 1 (1948) in start_year
pounds_start = interp1( UK2019.year, UK2019.pounds_1948, start_year + 1948 )/100;

%Apply inflation to initial_fund_value (in k)
initial_fund_value_inflated =...
    initial_fund_value * pounds_years/pounds_start ;

%Plot portfolio value vs time
figure('color',[1 1 1],'name','portfolio value')
t_years = (0:num_days-1)/365;
plot(t_years, portfolio_value1,...
    t_years, portfolio_value2,...
    t_years, portfolio_value3,...
    t_years, portfolio_value4,...
    t_years,initial_fund_value_inflated,'k'  );
xlabel('time /years','fontsize',fsize);
ylabel('Portfolio value /k','fontsize',fsize);
title(['Growth of ',num2str(initial_fund_value*1000),' from ',...
    start_date,' to ',end_date],'fontsize',fsize);
set(gca,'fontsize',fsize);
legend({legend_1,legend_2,legend_3,legend_4,'Inflation'},...
    'fontsize',fsize,'location','northwest');
grid on;
axis tight
ylim([0,value_max]);
print( gcf, ['portfolio value ',model_run_str,'.png'],'-r300','-dpng');

%Plot portfolio cash vs time
clf;
t_years = (0:num_days-1)/365;
plot( t_years, portfolio_cash1,...
    t_years, portfolio_cash2,...
t_years, portfolio_cash3,...
t_years, portfolio_cash4 );
xlabel('time /years','fontsize',fsize);
ylabel('Portfolio cash /k','fontsize',fsize);
title(['Portfolio cash ',...
    start_date,' to ',end_date],'fontsize',fsize);
set(gca,'fontsize',fsize);
legend({legend_1,legend_2,legend_3,legend_4},'fontsize',fsize,'location','northwest');
grid on;
axis tight
ylim([0,cash_max]);
print( gcf, ['portfolio cash ',model_run_str,'.png'],'-r300','-dpng');
close(gcf);

%Display completion message
disp(' ')
disp([' FTSE100 investment simulation completed in ',num2str(toc(t0)),'s.'])
disp(' ');

%%

%Run zbroker simulation
function [P, portfolio_value, portfolio_cash] = run_simulation(initial_fund_value)
global p s L T zsell zbuy

%Load FTSE data
disp(' ... Loading FTSE100 statistics')
load FTSE100_stats;

%Obtain maxium number of stocks and number of days of FTSE record
[num_stocks,num_days] = size(S_interp);

%Initialize global arrays p (portfolio) and s (stocks) which will be
%updated each day.
p.value = initial_fund_value;
p.cash = initial_fund_value;
p.shares = []; p.stocks = []; p.stockprice = []; p.z = [];
s.stocks = 1:num_stocks; s.stockprice = S_interp(:,1).';
s.z = S_z(:,1).';

%Remove any stocks are not in the FTSE (NaN entries)
remove_nan_stocks;

%Initial purchase of stocks
disp(' ... Calculating initial stock purchase ' );
initial_stock_purchase;

%Step through FTSE historical data, and buy or sell depending on z value
dd = 0;
disp([' ... Running investment simulation from ',start_date,' to ',end_date] );
for d=1:num_days
      
    %Display day number to command window
    dd = dd + 1;
    if dd==1000;
        disp(['     Day ',num2str(d),' of ',num2str(num_days),...
            '. Cash (k)=',num2str(p.cash,3),...
            ', portfolio value (k)=',num2str(p.value,3)])
        dd = 0;
    end
    
    %Update stocks
    s.stocks = 1:num_stocks;
    s.stockprice = S_interp(:,d).';
    s.z = S_z(:,d).';
    
    %Remove any stocks are not in the FTSE (NaN entries)
    %Sell shares automatically if they have left the FTSE
    remove_nan_stocks;
    
    %Update portfolio values
    update_portfolio;
    
    %Sell shares based upon z value
    sell_shares;
    
    %Buy shares based upon z value
    buy_shares;
    
    %Store p in time history of portfolio
    P(d) = p;
    portfolio_value(d) = p.value;
    portfolio_cash(d) = p.cash;
end

%%

%Update portfolio with new stock price and z value
function update_portfolio
global p s
for n=1:numel(p.stocks)
    i = find( s.stocks == p.stocks(n) );
    p.stockprice(n) = s.stockprice(i);
    p.z(n) = s.z(i);
end

%%

%Determine if any stocks are not in the FTSE (NaN entries)
%Sell shares automatically if they have left the FTSE
function remove_nan_stocks
global p s L

%Find NaN stock numbers (i.e. those no longer in FTSE)
i = s.stocks( isnan(s.stockprice) );

%Step through portfolio stocks and sell any that are now NaN
sell = [];
if ~isempty( p.stocks )
    for n=1:length(p.stocks)
        if ismember( p.stocks(n), i )
            sell = [sell,n];
            p.cash = p.cash + (1-L/100)*p.stockprice(n)*p.shares(n)/1e5;
        end
    end
end
p.stocks(sell) = [];
p.stockprice(sell) = [];
p.shares(sell) = [];
p.z(sell) = [];
p.value = p.cash + sum( p.stockprice.*p.shares/1e5 );

%Remove NaN stocks from s
s.stocks(i) = [];
s.stockprice(i) = [];
s.z(i) = [];

%%

%Initial purchase of stocks
function initial_stock_purchase
global p s L

%Spend entire cash fund on stocks, equal amount per stock
Ns = numel(s.stocks);
pence_per_stock = (p.cash*1e5/Ns)/( 1 + L/100 );
for n=1:Ns
    p.stocks(n) = s.stocks(n);
    p.shares(n) = floor( pence_per_stock/s.stockprice(n) );
    p.stockprice(n) = s.stockprice(n);
    p.z(n) = s.z(n);
    p.cash = p.cash - (1+L/100)*p.shares(n)*s.stockprice(n)/1e5;
end

%Recalculate fund value
p.value = p.cash + sum( p.shares.*p.stockprice/1e5 );

%%

%Buy shares based upon z value
function buy_shares
global p s L zbuy

%If z value of any held stocks is below zbuy, buy shares
buy = s.stocks( s.z < zbuy );
if ~isempty(buy)
    
    %Determine pence per stock that can be sold
    Ns = numel(buy);
    pence_per_stock = (p.cash*1e5/Ns)/( 1 + L/100 );
    
    %Sell stocks, and transfer sale proceeds to p.cash
    for n=1:length(buy)
        
        %What element of s.stocks corresponds to buy(n)?
        j = find( s.stocks==buy(n) );
        
        %Calculate how many shares can be bought
        num_shares = floor( pence_per_stock/s.stockprice(j) );
        
        %Is stock buy(n) already in the portfolio? If not, add it
        i = find( p.stocks==buy(n) );
        if isempty(i)       
            %Buy shares and update p.cash
            p.cash = p.cash -...
                (1+L/100)*s.stockprice(j)*num_shares/1e5;
            
            %Add new stocks to portfolio
            p.stocks = [p.stocks,buy(n)];
            p.shares = [p.shares,num_shares];
            p.stockprice = [p.stockprice,s.stockprice(j)];
            p.z = [p.z,s.z(j)];
        else
            %Buy shares and update p.cash
            p.cash = p.cash -...
                (1+L/100)*p.stockprice(i)*num_shares/1e5;
            
            %Add shares to existing stock in portfolio
            p.shares(i) = p.shares(i) + num_shares;
        end
    end
end

%Recalculate fund value
p.value = p.cash + sum( p.shares.*p.stockprice/1e5 );

%%

%Sell shares based upon z value
function sell_shares
global p L T zsell

%If z value of any held stocks is above zsell, sell shares
sell = find( p.z > zsell );
if ~isempty(sell)
    
    %Determine pence per stock that can be sold
    Ns = numel(sell);
    pence_per_stock = (T/100)*(p.value*1e5/Ns)/(1+L/100);
    
    %Sell stocks, and transfer sale proceeds to p.cash
    for n=1:length(sell)
        
        %Calculate how many shares can be sold
        num_shares = floor( pence_per_stock/p.stockprice( sell(n) ) );
        if num_shares > p.shares( sell(n) )
            num_shares = p.shares( sell(n) );
        end
        
        %Sell shares and update p.cash
        p.cash = p.cash +...
            (1-L/100)*p.stockprice( sell(n) )*num_shares/1e5;
        
        %Remove shares from portfolio
        p.shares( sell(n) ) = p.shares( sell(n) ) - num_shares;
    end
end

%If all shares are sold, remove stocks from portfolio
rm = [];
for n=1:numel(p.stocks)
    if p.shares(n)==0;
        rm = [rm,n];
    end
end
p.stocks(rm) = [];
p.stockprice(rm) = [];
p.shares(rm) = [];
p.z(rm) = [];

%Recalculate fund value
p.value = p.cash + sum( p.shares.*p.stockprice/1e5 );

%%

%Scale by inflation value at a given year
function y = scale_by_year( years, x, year, scale_at_year )
y = x*scale_at_year/interp1( years, x, year );

%End of code
